/**
    (C) Copyright 2012 Dominik Wagenfuehr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Game.hh"
#include "BaseBot.hh"
#include "AcceptBot.hh"
#include "DeclineBot.hh"
#include "VariableBot.hh"

#include <iostream>
#include <sstream>

// Startet das Programm.
bool Game::start( const std::string& strategy )
{
    BaseBot* botP = 0;

    // Je nach Strategie erstellen wir einen anderen Bot.
    if ( std::string("accept")  == strategy )
    {
        botP = new AcceptBot();
    }
    else if ( std::string("decline") == strategy )
    {
        botP = new DeclineBot();
    }
    else if ( std::string("variable") == strategy )
    {
        botP = new VariableBot();
    }

    bool retValue = false;

    // Wurde ein Bot erstellt?
    if ( botP )
    {
        // Bot ist verfuegbar, Strategie ist also gueltig.
        retValue = true;

        std::string input;
        int value = 0;
        do {
            // Eingabe vom Benutzer lesen
            input = readInput();

            // Eingabe in Zahl wandeln
            value = parseInput( input );

            if ( 0 <= value && value <= 1000 )
            {
                if ( botP->acceptOffer( value) )
                {
                    std::cout << "Bot nimmt Angebot an." << std::endl;
                }
                else
                {
                    std::cout << "Bot lehnt Angebot ab." << std::endl;
                }
            }
            // wiederhole, bis Wert ungueltig ist
        } while ( 0 <= value && value <= 1000 );

        // Das Spiel ist zu Ende, wir geben die Punkte aus.
        std::cout << "Gesammelte Punkte: " << botP->getPoints() << std::endl;
    }
    else
    {
        std::cout << "Strategie unbekannt oder anderer Fehler." << std::endl;
    }

    return retValue;
}

// Liest eine Eingabe vom Benutzer.
std::string Game::readInput() const
{
    std::string input;
    std::cout << "Zahl eingeben: ";
    getline(std::cin, input);
    return input;
}

// Interpretiert die Benutzereingabe.
int Game::parseInput( const std::string& input ) const
{
    // String in Integer konvertieren
    int value = 0;
    std::istringstream in(input);
    in >> value;
    return value;
}

